@@ -24,11 +24,11 @@ class GlancesCursesBrowser(_GlancesCurses):
         super().__init__(args=args)
 
         _colors_list = {
-            'UNKNOWN': self.no_color,
-            'SNMP': self.default_color2,
-            'ONLINE': self.default_color2,
-            'OFFLINE': self.ifCRITICAL_color2,
-            'PROTECTED': self.ifWARNING_color2,
+            'UNKNOWN': self.colors_list['DEFAULT'],
+            'SNMP': self.colors_list['OK'],
+            'ONLINE': self.colors_list['OK'],
+            'OFFLINE': self.colors_list['CRITICAL'],
+            'PROTECTED': self.colors_list['WARNING'],
         }
         self.colors_list.update(_colors_list)
 
@@ -249,15 +249,19 @@ class GlancesCursesBrowser(_GlancesCurses):
         screen_x = self.screen.getmaxyx()[1]
         screen_y = self.screen.getmaxyx()[0]
         stats_max = screen_y - 3
-        stats_len = len(stats)
-
         self._page_max_lines = stats_max
-        self._page_max = int(math.ceil(stats_len / stats_max))
-        # Init position
-        x = 0
-        y = 0
+        self._page_max = int(math.ceil(len(stats) / stats_max))
 
-        # Display top header
+        # Display header
+        x, y = self.__display_header(stats, 0, 0, screen_x, screen_y)
+
+        # Display Glances server list
+        # ================================
+        return self.__display_server_list(stats, x, y, screen_x, screen_y)
+
+    def __display_header(self, stats, x, y, screen_x, screen_y):
+        stats_len = len(stats)
+        stats_max = screen_y - 3
         if stats_len == 0:
             if self.first_scan and not self.args.disable_autodiscover:
                 msg = 'Glances is scanning your network. Please wait...'
@@ -268,8 +272,8 @@ class GlancesCursesBrowser(_GlancesCurses):
             msg = 'One Glances server available'
         else:
             msg = f'{stats_len} Glances servers available'
-        if self.args.disable_autodiscover:
-            msg += ' (auto discover is disabled)'
+        # if self.args.disable_autodiscover:
+        #     msg += ' (auto discover is disabled)'
         if screen_y > 1:
             self.term_window.addnstr(y, x, msg, screen_x - x, self.colors_list['TITLE'])
 
@@ -282,33 +286,51 @@ class GlancesCursesBrowser(_GlancesCurses):
             msg = f'{page_lines} servers displayed.({self._current_page + 1}/{self._page_max}) {status_count}'
             self.term_window.addnstr(y + 1, x, msg, screen_x - x)
 
-        if stats_len == 0:
+        return x, y
+
+    def __build_column_def(self, current_page):
+        """Define the column and it size to display in the browser"""
+        column_def = {'name': 16, 'ip': 15, 'status': 9, 'protocol': 8}
+
+        # Add dynamic columns
+        for server_stat in current_page:
+            for k, v in server_stat.items():
+                if k.endswith('_decoration'):
+                    column_def[k.split('_decoration')[0]] = 6
+        return column_def
+
+    def __display_server_list(self, stats, x, y, screen_x, screen_y):
+        if not stats:
+            # No server to display
             return False
 
-        # Display the Glances server list
-        # ================================
-
-        # Table of table
-        # Item description: [stats_id, column name, column size]
-        column_def = [
-            ['name', 'Name', 16],
-            ['alias', None, None],
-            ['load_min5', 'LOAD', 6],
-            ['cpu_percent', 'CPU%', 5],
-            ['mem_percent', 'MEM%', 5],
-            ['status', 'STATUS', 9],
-            ['ip', 'IP', 15],
-            # ['port', 'PORT', 5],
-            ['hr_name', 'OS', 16],
-        ]
-        y = 2
+        stats_list = self._get_stats(stats)
+        start_line = self._page_max_lines * self._current_page
+        end_line = start_line + self.get_pagelines(stats_list)
+        current_page = stats_list[start_line:end_line]
+        column_def = self.__build_column_def(current_page)
 
         # Display table header
+        stats_max = screen_y - 3
+        y = 2
         xc = x + 2
-        for cpt, c in enumerate(column_def):
-            if xc < screen_x and y < screen_y and c[1] is not None:
-                self.term_window.addnstr(y, xc, c[1], screen_x - x, self.colors_list['BOLD'])
-                xc += c[2] + self.space_between_column
+        # First line (plugin name)
+        for k, v in column_def.items():
+            k_split = k.split('_')
+            if len(k_split) == 1:
+                xc += v + self.space_between_column
+                continue
+            if xc < screen_x and y < screen_y and v is not None:
+                self.term_window.addnstr(y, xc, k_split[0].upper(), screen_x - x, self.colors_list['BOLD'])
+                xc += v + self.space_between_column
+        xc = x + 2
+        y += 1
+        # Second line (for item/key)
+        for k, v in column_def.items():
+            k_split = k.split('_')
+            if xc < screen_x and y < screen_y and v is not None:
+                self.term_window.addnstr(y, xc, ' '.join(k_split[1:]).upper(), screen_x - x, self.colors_list['BOLD'])
+                xc += v + self.space_between_column
         y += 1
 
         # If a servers has been deleted from the list...
@@ -317,31 +339,12 @@ class GlancesCursesBrowser(_GlancesCurses):
             # Set the cursor position to the latest item
             self.cursor = len(stats) - 1
 
-        stats_list = self._get_stats(stats)
-        start_line = self._page_max_lines * self._current_page
-        end_line = start_line + self.get_pagelines(stats_list)
-        current_page = stats_list[start_line:end_line]
-
         # Display table
         line = 0
-        for v in current_page:
+        for server_stat in current_page:
             # Limit the number of displayed server (see issue #1256)
             if line >= stats_max:
                 continue
-            # Get server stats
-            server_stat = {}
-            for c in column_def:
-                try:
-                    server_stat[c[0]] = v[c[0]]
-                except KeyError as e:
-                    logger.debug(f"Cannot grab stats {c[0]} from server (KeyError: {e})")
-                    server_stat[c[0]] = '?'
-                # Display alias instead of name
-                try:
-                    if c[0] == 'alias' and v[c[0]] is not None:
-                        server_stat['name'] = v[c[0]]
-                except KeyError:
-                    pass
 
             # Display line for server stats
             cpt = 0
@@ -354,11 +357,24 @@ class GlancesCursesBrowser(_GlancesCurses):
 
             # Display the line
             xc += 2
-            for c in column_def:
-                if xc < screen_x and y < screen_y and c[1] is not None:
+            for k, v in column_def.items():
+                if xc < screen_x and y < screen_y:
                     # Display server stats
-                    self.term_window.addnstr(y, xc, format(server_stat[c[0]]), c[2], self.colors_list[v['status']])
-                    xc += c[2] + self.space_between_column
+                    value = server_stat.get(k, '?')
+                    if isinstance(value, float):
+                        value = round(value, 1)
+                    if k == 'name' and 'alias' in server_stat and server_stat['alias'] is not None:
+                        value = server_stat['alias']
+                    decoration = self.colors_list.get(
+                        server_stat[k + '_decoration'].replace('_LOG', '')
+                        if k + '_decoration' in server_stat
+                        else self.colors_list[server_stat['status']],
+                        self.colors_list['DEFAULT'],
+                    )
+                    if k == 'status':
+                        decoration = self.colors_list[server_stat['status']]
+                    self.term_window.addnstr(y, xc, format(value), v, decoration)
+                    xc += v + self.space_between_column
                 cpt += 1
             # Next line, next server...
             y += 1
